/*------------------------------------------------------------------------
  File: fw/conversion/12.03/fw-1203-after-06-fw-shipment_bl.p
  Doel: converteren van de FW-database van 12.02 naar 12.03
        
  Programmeur: Simon Roest
               (c)Softpak BV
  Datum: 18/06/2012

  Vanaf versie 12.03 zijn de gegevens van de Groupage Bookings en de
  Shipments gelijk getrokken. Dit betekent dat aan de Shipments ook
  gegevens voor het B/L-record toegevoegd.
  Dit conversieprogramma zal initieel voor alle bestaande Shipments
  het benodigde B/L-record en de onderliggende adressen aanmaken.
------------------------------------------------------------------------*/
{system/conversion/conversion_progress.i}
DEFINE VARIABLE cProgram              AS CHARACTER NO-UNDO.
DEFINE VARIABLE cAction               AS CHARACTER NO-UNDO.
DEFINE VARIABLE iDone                 AS INTEGER   NO-UNDO.
DEFINE VARIABLE iModulo               AS INTEGER   NO-UNDO.

DEFINE VARIABLE cRoleString           AS CHARACTER NO-UNDO.
DEFINE VARIABLE iLoop                 AS INTEGER   NO-UNDO.

ASSIGN cProgram =  "fw-1203-after-06-fw-shipment_bl.p"
       iModulo  =  100.  /* om het record info tonen */

FORM "Program  :" AT 1 cProgram   NO-LABEL  FORMAT "x(50)"    AT 12  SKIP
     "Action   :" AT 1 cAction    NO-LABEL  FORMAT "x(50)"    AT 12  SKIP
     "Done     :" AT 1 iDone      NO-LABEL  FORMAT "9999999"  AT 12  SKIP   
     WITH FRAME a.

DISABLE TRIGGERS FOR LOAD OF fw-load_house.
DISABLE TRIGGERS FOR LOAD OF fw-load_house_address.

FUNCTION spVulBox RETURNS CHARACTER
  ( INPUT pcCompanyCode AS CHARACTER,
    INPUT pcAddressnr   AS CHARACTER )  FORWARD.

ASSIGN cAction = "Aanmaken Shipment B/L".
       iDone   = 0.
RUN DisplayProgress (cProgram, cAction, '', 0, iDone).

FOR EACH fw-load NO-LOCK:

  IF NOT CAN-FIND(fw-load_house WHERE
                  fw-load_house.companycode = fw-load.companycode AND
                  fw-load_house.loadnr      = fw-load.loadnr
                  NO-LOCK)
  THEN
  DO TRANSACTION:

    FIND fw-parameter WHERE
         fw-parameter.companycode = fw-load.companycode
         NO-LOCK NO-ERROR.
    FIND fw-parameter-2 WHERE
         fw-parameter-2.companycode = fw-load.companycode
         NO-LOCK NO-ERROR.
    /* BL_place,ind_bl_numbering,ind_notify,notify_text,ind_shipper,ind_hbl_customer */

    /* Zoek de standaard signature op */
    FIND FIRST fw-signature WHERE
               fw-signature.companycode = fw-load.companycode AND
               fw-signature.standaard   = TRUE
               NO-LOCK NO-ERROR.

    CREATE fw-load_house.

    /* ===================================================== */  
    /* Leg de sleutel en enkele andere bekende gegevens vast */
    /* ===================================================== */  
    ASSIGN fw-load_house.companycode   = fw-load.companycode
           fw-load_house.loadnr        = fw-load.loadnr
           /* ============================= */
           /* Default betaalwijze: pre-paid */
           /* ============================= */
           fw-load_house.TYPE_payable  = "P":U
           fw-load_house.date-create   = TODAY
           fw-load_house.user-create   = "SOFTPAK"
           fw-load_house.date-modified = TODAY
           fw-load_house.user-modified = "SOFTPAK".

    ASSIGN fw-load_house.BL-place      = fw-parameter.BL_place
           fw-load_house.BL-number     = (IF fw-parameter.ind_bl_numbering = TRUE
                                           THEN STRING(fw-load.loadnr,"99999999")
                                           ELSE "")
           fw-load_house.BL-ref        = fw-load.lot_reference
           fw-load_house.freight       = fw-parameter.BL_place.

    IF AVAILABLE fw-signature
    THEN ASSIGN fw-load_house.signature = fw-signature.signature.

    IF fw-parameter.ind_notify = TRUE
    THEN ASSIGN cRoleString = "NOT".
    IF fw-parameter.ind_shipper = TRUE
    THEN ASSIGN cRoleString = cRoleString
                            + (IF cRoleString = "" THEN "" ELSE ",")
                            + "SHP".
    IF fw-parameter-2.ind_hbl_customer = TRUE
    THEN ASSIGN cRoleString = cRoleString
                            + (IF cRoleString = "" THEN "" ELSE ",")
                            + "CUS".

    DO iLoop = 1 TO NUM-ENTRIES(cRoleString):

      AddressBlock:
      FOR EACH fw-bl_role WHERE 
               fw-bl_role.typecode = ENTRY(iLoop, cRoleString)
               NO-LOCK:
  
        IF fw-bl_role.ind_inactive = TRUE
        THEN NEXT AddressBlock.
  
        CREATE fw-load_house_address.
  
        ASSIGN fw-load_house_address.companycode   = fw-load_house.companycode
               fw-load_house_address.loadnr        = fw-load_house.loadnr
               fw-load_house_address.role          = fw-bl_role.rolecode
               fw-load_house_address.addressnr     = fw-load.customernr
               fw-load_house_address.date-create   = TODAY
               fw-load_house_address.user-create   = "SOFTPAK"
               fw-load_house_address.date-modified = TODAY
               fw-load_house_address.user-modified = "SOFTPAK".
  
        ASSIGN fw-load_house_address.addressbox    = (IF fw-bl_role.typecode = "NOT"
                                                      THEN fw-parameter.notify_text
                                                      ELSE spVulBox(fw-load.companycode,
                                                                    STRING(fw-load.customernr))).

        FIND aa-address WHERE
             aa-address.addressnr = fw-load_house_address.addressnr
             NO-LOCK NO-ERROR.
       
        IF AVAILABLE aa-address
        THEN ASSIGN fw-load_house_address.name_1 = SUBSTRING(aa-address.name_1, 1, 35)
                    fw-load_house_address.name_2 = SUBSTRING(aa-address.name_2, 1, 35).
        ELSE ASSIGN fw-load_house_address.name_1 = SUBSTRING(ENTRY(1, fw-load_house_address.addressbox, CHR(10)),1,35).
      END.
    END.
  END.

  iDone = iDone  + 1.
  IF iDone MODULO iModulo = 0 THEN RUN DisplayProgress (cProgram, cAction, '', 0, iDone).
END.

RUN DisplayProgress (cProgram, cAction, '', 0, iDone).

/* ******************** F U N C T I E S ******************** */
FUNCTION spVulBox RETURNS CHARACTER
  ( INPUT pcCompanyCode AS CHARACTER,
    INPUT pcAddressnr   AS CHARACTER ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/
  DEFINE VARIABLE cReturnValue  AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE cParameters   AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cFieldString  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cValueString  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cCurrentField AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cCurrentValue AS CHARACTER NO-UNDO.
  DEFINE VARIABLE iLoop         AS INTEGER   NO-UNDO.
  
  DEFINE BUFFER baa-address   FOR aa-address.
  DEFINE BUFFER bfw-parameter FOR fw-parameter.

  FIND bfw-parameter WHERE
       bfw-parameter.companycode = pcCompanyCode
       NO-LOCK NO-ERROR.
  
  /*===================== */
  /* Zoek het adres erbij */
  /*===================== */
  FIND baa-address WHERE
       baa-address.addressnr = INTEGER(pcAddressnr)
       NO-LOCK NO-ERROR.
  
  /* ================================================= */
  /* Als er geen adres is, hoeven we ook niets te doen */
  /* ================================================= */
  IF AVAILABLE baa-address
  THEN
  DO:
    /* =================== */
    /* Zoek het land erbij */
    /* =================== */
    FIND aa-country WHERE
         aa-country.countrycode = baa-address.countrycode
         NO-LOCK NO-ERROR.
         
    FIND fw-address_format WHERE
         fw-address_format.companycode = pcCompanyCode AND
         fw-address_format.countrycode = baa-address.countrycode
         NO-LOCK NO-ERROR.
  
    IF NOT AVAILABLE fw-address_format OR
       fw-address_format.ind_inactive = TRUE
    THEN
      FIND fw-address_format WHERE
           fw-address_format.companycode = pcCompanyCode AND
           fw-address_format.countrycode = bfw-parameter.DEFAULT_addressformat
           NO-LOCK NO-ERROR.
  
    IF AVAILABLE fw-address_format AND
       fw-address_format.ind_inactive = FALSE
    THEN
    DO:
      ASSIGN cFieldString = "addressnr"   + CHR(4)
                          + "shortname"   + CHR(4)
                          + "name_1"      + CHR(4)
                          + "name_2"      + CHR(4)
                          + "address_1"   + CHR(4)
                          + "address_2"   + CHR(4)
                          + "housenr"     + CHR(4)
                          + "city"        + CHR(4)
                          + "state"       + CHR(4)
                          + "zipcode"     + CHR(4)
                          + "countrycode" + CHR(4)
                          + "country"     + CHR(4)
                          + "telephonenr" + CHR(4)
                          + "faxnr"       + CHR(4)
                          + "telexnr"     + CHR(4)
                          + "emailaddress"
             cValueString = STRING(baa-address.addressnr,"99999999") + CHR(4)
                          + baa-address.shortname                    + CHR(4)
                          + baa-address.NAME_1                       + CHR(4)
                          + baa-address.NAME_2                       + CHR(4)
                          + baa-address.address_1                    + CHR(4)
                          + baa-address.address_2                    + CHR(4)
                          + baa-address.housenr                      + CHR(4)
                          + baa-address.city                         + CHR(4)
                          + baa-address.state                        + CHR(4)
                          + baa-address.zipcode                      + CHR(4)
                          + baa-address.countrycode                  + CHR(4)
                          + (IF AVAILABLE aa-country
                             THEN aa-country.des
                             ELSE "")                                + CHR(4)
                          + baa-address.telephonenr                  + CHR(4)
                          + baa-address.faxnr                        + CHR(4)
                          + baa-address.telexnr                      + CHR(4)
                          + baa-address.emailaddress
             cReturnValue = fw-address_format.format_string.
  
      /* ===================================================== */
      /* Aangegeven alles hoofdletter ? Dan alles hoofdletters */
      /* ===================================================== */
      IF fw-address_format.ind_capitals = TRUE
      THEN
        ASSIGN cValueString = CAPS(cValueString).
  
      DO iLoop = 1 TO NUM-ENTRIES(cFieldString, CHR(4)):
        ASSIGN cCurrentField = "<" + ENTRY(iLoop, cFieldString, CHR(4)) + ">"
               cCurrentValue = ENTRY(iLoop, cValueString, CHR(4))
               .
  
        ASSIGN cReturnValue = REPLACE(cReturnValue, cCurrentField, cCurrentValue).
      END.
  
      DO iLoop = 1 TO NUM-ENTRIES(cReturnvalue, CHR(10)):
        IF ENTRY(iLoop, cReturnvalue, CHR(10)) = "" AND
           fw-address_format.ind_omit_empty = TRUE
        THEN
          ASSIGN ENTRY(iLoop, cReturnvalue, CHR(10)) = "XXXQQQXXX".
        ELSE
          IF ENTRY(iLoop, cReturnvalue, CHR(10)) <> ""
          THEN
            ASSIGN cReturnValue = REPLACE(cReturnValue, ENTRY(iLoop, cReturnvalue, CHR(10)), LEFT-TRIM(ENTRY(iLoop, cReturnvalue, CHR(10)))).
      END.
  
      ASSIGN cReturnValue = REPLACE(cReturnValue, "XXXQQQXXX" + CHR(10), "").
    END.
    ELSE
    DO:
      ASSIGN cReturnValue = (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_name_1 <> ""
                                THEN baa-address.h_name_1 + CHR(10)
                                ELSE "")
                             ELSE
                               (IF baa-address.name_1 <> ""
                                THEN baa-address.name_1 + CHR(10)
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_name_2 <> ""
                                THEN baa-address.h_name_2 + CHR(10)
                                ELSE "")
                             ELSE
                               (IF baa-address.name_2 <> ""
                                THEN baa-address.name_2 + CHR(10)
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_address_1 <> "" OR
                                   baa-address.h_housenr   <> ""
                                THEN baa-address.h_address_1
                                     + " " + baa-address.h_housenr + CHR(10)
                                ELSE "")
                             ELSE
                               (IF baa-address.address_1 <> "" OR
                                   baa-address.housenr   <> ""
                                THEN baa-address.address_1
                                     + " " + baa-address.housenr + CHR(10)
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_address_2 <> ""
                                THEN baa-address.h_address_2 + CHR(10)
                                ELSE "")
                             ELSE
                               (IF baa-address.address_2 <> ""
                                THEN baa-address.address_2 + CHR(10)
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_zipcode <> ""
                                THEN baa-address.h_zipcode
                                ELSE "")
                             ELSE
                               (IF baa-address.zipcode <> ""
                                THEN baa-address.zipcode
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN
                               (IF baa-address.h_zipcode <> "" AND
                                   baa-address.h_city    <> ""
                                THEN "  ":U
                                ELSE "":U)
                             + (IF baa-address.h_city <> ""
                                THEN baa-address.h_city + CHR(10)
                                ELSE "")
                             ELSE
                               (IF baa-address.zipcode <> "" AND
                                   baa-address.city    <> ""
                                THEN "  ":U
                                ELSE "":U)
                             + (IF baa-address.city <> ""
                                THEN baa-address.city + CHR(10)
                                ELSE ""))
                          + (IF baa-address.ind_alt_house_address = TRUE
                             THEN ""
                             ELSE
                               (IF baa-address.state <> ""
                                THEN baa-address.state + CHR(10)
                                ELSE ""))
                          + (IF AVAILABLE aa-country
                             THEN aa-country.des
                             ELSE "")                         
                          .
    END.
  END.
  
  RETURN cReturnValue.
END FUNCTION.
